/* feat.c -- common routines for working with "features" files
 *           (exons, repeats, annotations, underlays, etc.)
 *
 * $Revision: 1.3 $$Date: 2001/05/04 01:41:02 $
 */

#include "feat.h"
#include "util.h"

static bool is_exon_punct (char *head, char *p);

/* extract interval endpoints and rest of buffer */
bool feat_parse_line (char *buf, char *head, int *a1, char *mid, int *a2,
                      char **tail) {
	int n;
	char *p;

	if (starts (skip_ws (buf), "#"))
		return 0;
	if (sscanf (buf, "%[^0-9]%d%[ \t]%d%n", head, a1, mid, a2, &n) != 4) {
		*head = '\0';
		if (sscanf (buf, "%d%[ \t]%d%n", a1, mid, a2, &n) != 3) {
			return 0;
		}
	}
	*tail = buf + n;

	/* check that extracted numbers are not parts of other words */
	if (*head) {
		p = head + strlen (head) - 1;
		if (!isspace (*p) && !is_exon_punct (head, p))
			return 0;
	}
	if (!isspace (**tail)) {    /* tail contains at least '\n' */
		return 0;
	}

	return 1;
}

/* identify special exons-file punctuation */
static bool is_exon_punct (char *head, char *p) {
	return (p == skip_ws (head) &&
			(*p == '>' || *p == '<' || *p == '|' || *p == '+'));
}

/* try to determine filetype from input line */
void feat_guess_filetype (char *buf, char *filetype) {
	char *p = skip_ws (buf);

	if (*p == '#')
		return;
	else if (*p == '>' || *p == '<' || *p == '|') {
		fprintf (stderr, "This appears to be an exons file.\n");
		*filetype = 'e';
	}
	else if (starts (p, "%define") && strstr (p, "annotation")) {
		fprintf (stderr, "This appears to be an annotations file.\n");
		*filetype = 'a';
	}
	else if (starts (p, "%:repeats") || strstr (p, "Right") ||
			strstr (p, "Left")) {
		fprintf (stderr, "This appears to be a repeats file.\n");
		*filetype = 'r';
	}
}

/* reverse direction indicators in input line; TAIL ARRAY MUST NOT BE FULL */
 void feat_reverse_dir (char *head, char *tail, char filetype) {
	char *p = skip_ws (head);

	if (*p == '#')
		return;
	else if (filetype == 'e') {
		if (*p == '>')
			*p = '<';
		else if (*p == '<')
			*p = '>';
	}
	else if (filetype == 'r') {
		if ((p = strstr (tail, "Right"))) {
			memmove (p+4, p+5, strlen (p+5) + 1);
			memcpy (p, "Left", 4);
		}
		else if ((p = strstr (tail, "Left"))) {
			memmove (p+5, p+4, strlen (p+4) + 1);   /* need room here */
			memcpy (p, "Right", 5);
		}
	}
}

